/***************************************************************

        Project     : OBONO
        File        : "main.c"
        Author      : Shunsuke Kudo
        Create Date : 2005.02/05
        Update Date : 2005.02/05

***************************************************************/





/*==============================================================
        Inclide Header Files
==============================================================*/
#include <stdlib.h>
#include <SDL.h>
#ifdef WIN32
#include <windows.h>
#endif


/*==============================================================
        Definitions
==============================================================*/
#define WIN_WIDTH   320
#define WIN_HEIGHT  240
#define OBJECT_CNT  32

/*==============================================================
        Functions
==============================================================*/
void    render(void);
void    init_object(void);
void    move_object(void);

/*==============================================================
        Variables
==============================================================*/
typedef struct {
    double  x, y, vx, vy;
} OBJECT;

SDL_Surface     *screen;
SDL_Surface     *img_obono;
OBJECT          object[OBJECT_CNT];





/*==============================================================
        Main Function
==============================================================*/
int     main(int argc, char *argv[])
{
    SDL_Event   event;
    int         flag = SDL_SWSURFACE;

#ifdef WIN32
    if (MessageBox(NULL, "Full Scrren?", "Notice", MB_YESNO | MB_DEFBUTTON2) == IDYES)
    {
        flag |= SDL_FULLSCREEN; 
    }
#endif

    /*--------------*/
    /*  Initialize  */
    /*--------------*/

    /*  Init SDL    */
    if (SDL_Init(SDL_INIT_VIDEO) < 0)
    { 
        fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
        exit(1);
    }
    atexit(SDL_Quit);

    /*  Create Windows  */
    SDL_WM_SetCaption("OBONO", NULL);
    screen = SDL_SetVideoMode(WIN_WIDTH, WIN_HEIGHT, 32, flag);
    if (screen == NULL)
    { 
        fprintf(stderr, "Unable to set %dx%d video: %s\n", WIN_WIDTH, WIN_HEIGHT, SDL_GetError());
        exit(1);
    }

    /*  Load OBONO Image    */
    img_obono = SDL_LoadBMP("obono.bmp");
    if (screen == NULL)
    { 
        fprintf(stderr, "Unable to load \"obono.bmp\": %d\n", SDL_GetError());
        exit(1);
    }
    SDL_SetColorKey(img_obono, SDL_SRCCOLORKEY, SDL_MapRGB(img_obono->format, 0xFF, 0xFF, 0xFF));

    /*  Init Objects    */
    init_object();


    /*--------------*/
    /*  Main Loop   */
    /*--------------*/

    while (1)
    {
        /*  Move Objects & Draw Screen  */
        move_object();
        render();

        while (SDL_PollEvent(&event))
        {
            switch (event.type)
            {
                case SDL_MOUSEMOTION:
                    object[OBJECT_CNT - 1].x = event.motion.x;
                    object[OBJECT_CNT - 1].y = event.motion.y;
                    break;

                case SDL_KEYDOWN:
                   break;

                case SDL_KEYUP:
                    if (event.key.keysym.sym == SDLK_ESCAPE)
                        return 0;
                    break;

                case SDL_QUIT:
                    return 0;
            }
        }

        SDL_Delay(20);
    }

    return 0;
}


/*==============================================================
        Init Objects
==============================================================*/
void    init_object(void)
{   
    int     i;
    OBJECT  *ptr;

    srand(SDL_GetTicks());

    for (i = 0, ptr = object; i < OBJECT_CNT; i++, ptr++) {
        ptr->x  = (double)(rand() % WIN_WIDTH);
        ptr->y  = (double)(rand() % WIN_HEIGHT);
        ptr->vx = 0.0;
        ptr->vy = 0.0;
    }

    return;
}


/*==============================================================
        Move Objects
==============================================================*/
void    move_object(void)
{   
    int     i, j;
    double  dx, dy, dist;
    OBJECT  *ptr1, *ptr2;

    for (i = 0, ptr1 = object; i < OBJECT_CNT - 1; i++, ptr1++) {
        for (j = 0, ptr2 = object; j < OBJECT_CNT; j++, ptr2++) {
            if (i != j) {
                dx = ptr1 ->x - ptr2->x;
                dy = ptr1 ->y - ptr2->y;
                dist = dx * dx + dy * dy;
                if (dist > 1024.0) {
                    if (j == OBJECT_CNT - 1) {
                        dx = -dx;
                        dy = -dy;
                        dist /= 32.0;
                    }
                    ptr1->vx += dx / dist;
                    ptr1->vy += dy / dist;
                }
            }
        }
    }

    for (i = 0, ptr1 = object; i < OBJECT_CNT - 1; i++, ptr1++) {
        if (ptr1->x + ptr1->vx < 0 || ptr1->x + ptr1->vx > (double)WIN_WIDTH) {
            ptr1->vx = -(ptr1->vx) / 2.0;
        }
        if (ptr1->y + ptr1->vy < 0 || ptr1->y + ptr1->vy > (double)WIN_HEIGHT) {
            ptr1->vy = -(ptr1->vy) / 2.0;
        }
        ptr1->x += ptr1->vx;
        ptr1->y += ptr1->vy;
    }

    return;
}


/*==============================================================
        Rander Function
==============================================================*/
void    render(void)
{   
    int         i;
    OBJECT      *ptr;
    SDL_Rect    tgt;

    /*  Clear Screen    */
    SDL_FillRect(screen, NULL, 0xFFFFFF);

    /*  Draw to screen  */
    for (i = 0, ptr = object; i < OBJECT_CNT; i++, ptr++) {
        tgt.x = (Sint16)(ptr->x) - 8;
        tgt.y = (Sint16)(ptr->y) - 8;
        SDL_BlitSurface(img_obono, NULL, screen, &tgt);
    }

    /*  Lock surface if needed  */
    //if (SDL_LockSurface(screen) < 0) 
    //  return;

    /*  Unlock if needed    */
    //if (SDL_MUSTLOCK(screen)) 
    //  SDL_UnlockSurface(screen);

    /* Tell SDL to update the whole screen  */
    SDL_UpdateRect(screen, 0, 0, WIN_WIDTH, WIN_HEIGHT);

    return;
}


/*==============================================================
        END OF SOURCE
==============================================================*/
